/*

-Procedure  odt_example ( Example of computing MRO SHARAD ODT values )
 
-Abstract
 
   Compute and return a set of observation geometry parameters for MRO
   SHARAD Orbital Data table (ODT).
 
-Contact
 
   Boris Semenov, NAIF/JPL
   e-mail: Boris.Semenov@jpl.nasa.gov
   phone:  (818)-354-8136.
 
-Required_Reading
 
   Comments in the headers of the CSPICE functions used in this example
   SPK Required Reading (spk.req)
   PCK Required Reading (pck.req)
   FRAMES Required Reading (frames.req)
   NAIF_IDS Required Reading (naif_ids.req)
 
*/

   #include "SpiceUsr.h"
 
   void odt_example( double   start,
                     double   step,
                     long     n,
                     double * tlp_length,
                     double * tlp_angle,
                     double * radius,
                     double * rad_rate,
                     double * vel_tan )

/*
 
-Brief_I/O
 
   Variable        I/O  Description
   --------------  ---  --------------------------------------------
   start            I   Start ET, ephemeris seconds past J2000
   step             I   Time step, seconds
   n                I   Number of steps
   tlp_length       O   Array of TLP length values, km
   tlp_angle        O   Array of TLP angle values, milliradians
   radius           O   Array of radius values, km
   rad_rate         O   Array of radius rates, m/s
   vel_tan          O   Array of tangential velocities, m/s
 
-Detailed Input
 
   See Brief_I/O
   
-Detailed Output

   See Brief_I/O

-Exceptions
 
   1) If, at the time this routine is called, the SPICE data loaded 
      by the program are insufficient to permit computation of the
      requested parameters, the deficiency will be diagnosed by a
      CSPICE routines called by this function.
 
   2) If input time step is less than 1 second or greater than 600 
      seconds, the ODT_EXAMPLE(BADINPUTSTEP) error will be signaled.

   3) If input number of steps is less than 1, the 
      ODT_EXAMPLE(BADNUMBEROFSTEPS) error will be signaled.

-Files
 
   The following MRO SPICE kernels covering the time period of interest 
   should be loaded using CSPICE furnsh_c routine prior to calling 
   this routine:

      Generic LSK file
      Generic PCK file providing IAU 2000 Mars shape and rotation constants
      MRO spacecraft trajectory SPK file(s)
      Planetary Ephemeris SPK (if s/c SPK does not contain this data.)
      
-Particulars

   Notes regarding ODT parameter computations implemented by this
   routine (in no specific order):

      -  raduis, rad_rate, and vel_tan are computed from geometric 
         state (position and velocity) of MRO relative to Mars in 
         Mars body-fixed frame;

      -  rad_rate and vel_tan are computed by rotating velocity 
         component of that state into the orbital frame (+Z along 
         radius, +X in the direction of velocity, +Y normal to the 
         instantaneous orbital plane in Mars body-fixed frame) and 
         taking Z and X components of the rotated velocity.

      -  tlp_length is calculated by accumulating distances between 
         sub-spacecraft points (determined as the nearest point on 
         Mars ellipsoid) in Mars body-fixed frame, computed 
         at small fixed time increments between every pair of 
         output ODT times. The small increment is set by TLP_SUBSTEP
         below. Trial-n-error showed that it need not be smaller than
         0.1 second (but even 0.5 will do as the difference between TLP
         computed with 0.5 and 1.0 seconds over one orbit was under 1
         meter.)
      
      -  for convenience, tlp_angle is also calculated by accumulating
         angular separations between s/c position vectors in J2000
         inertial frame, computed at the same small fixed time
         increments as TLP_length.
 
-Examples
 
   See the "driver" program provided with this example.

-Restrictions
 
   See Files and Particulars sections.
 
-Literature_References
 
   None.
 
-Author_and_Institution
 
   B.V. Semenov    (JPL)
 
-Version
 
   -odt_example Version 1.0.0, 2005-MAY-12 (BVS)

      Initial version.

-Index_Entries
 
   None.

-& 
*/
 
{  /* Begin odt_example */
   
   /*
   Local constants.
   */
   #define  TLP_SUBSTEP     (double) 0.5 

   /*
   Local variables
   */
   SpiceDouble   alt;
   SpiceDouble   et;
   SpiceDouble   etprev;
   SpiceDouble   ettlp;
   SpiceDouble   lt;
   SpiceDouble   mat[3][3];
   SpiceDouble   scpos[3];
   SpiceDouble   scvel[3];
   SpiceDouble   scvelorb[3];
   SpiceDouble   spoint[3];
   SpiceDouble   spointdiff[3];
   SpiceDouble   spointp[3];
   SpiceDouble   spos[3];
   SpiceDouble   sposp[3];
   SpiceDouble   state[6];
   SpiceDouble   tlp_a;
   SpiceDouble   tlp_l;
   SpiceDouble   tlp_substep;

   SpiceInt      i;
   SpiceInt      j;
   SpiceInt      m;

   /*
   Make sure that time step and number of steps are acceptable.
   */
   if ( step < 1.0 || step > 600.0 )
   {
     setmsg_c ( "Step must be between 1 and 600 seconds. It was #." );
     errdp_c  ( "#",  step                                          );
     sigerr_c ( "ODT_EXAMPLE(BADINPUTSTEP)"                         );
   }

   if ( n < 1 )
   {
     setmsg_c ( "Number of steps must be at least 1. It was #."     );
     errint_c ( "#", n                                              );
     sigerr_c ( "ODT_EXAMPLE(BADNUMBEROFSTEPS)"                     );
   }

   /*
   Initialize accumulating TLP length and angle variables.
   */
   tlp_a = 0.0;
   tlp_l = 0.0;
   
   /*
   Main loop; from start+step through start+n*step. 
   */
   for ( i = 1; i <= n; i++ )
   {
     /*
     Compute ET time of this step.
     */
     et = start + i * step;

     /*
     Compute geometric state of MRO in Mars body-fixed frame.
     */
     spkezr_c( "MRO", et, "IAU_MARS", "NONE", "MARS", state, &lt );

     /*
     Copy position and velocity into separate vectors.
     */
     vequ_c  ( &state[0], scpos );
     vequ_c  ( &state[3], scvel );

     /*
     Compute distance from the center of Mars (it's the magnitude of
     the position vector) and save in the output array.
     */
     radius[i-1] = vnorm_c ( scpos );

     /*
     Construct transformation into the orbital reference frame (+Z
     along radius vector, +X in the direction of velocity, +Y is normal
     to the instantaneous orbit plane in Mars body-fixed frame).
     */
     twovec_c( scpos, 3, scvel, 1, mat );

     /*
     Rotate velocity vector into the orbital reference frame and assign
     output radial rate (the Z element of the s/c velocity in the
     orbital reference frame) and tangential velocity (the X element of
     the s/c velocity in the orbital reference frame.) Both are scaled
     to be m/s.
     */
     mxv_c ( mat, scvel, scvelorb );
     rad_rate[i-1] = scvelorb[2] * 1000.0;
     vel_tan[i-1]  = scvelorb[0] * 1000.0;

     /*
     Compute track length position (TLP) length and angle values. This
     requires a sub-loop with a few equally spaced in time iterations
     over the period from the previous point to this point. Start with
     computing the time of the previous point.
     */
     etprev = start + (i - 1) * step;

     /*
     Determine the number of iterations need in this sub-loop and
     adjust the sub-loop time step to match it.
     */
     m = (int) ( ( et - etprev ) / TLP_SUBSTEP ) ;
     if ( m <= 0   )
     {
       m = 1; 
     }
     tlp_substep = ( et - etprev ) / m;

     /*
     Go into the sub-loop computing TLP. But first, compute for etprev
     the sub-spacecraft point location (as the nearest point on Mars
     ellipsoid) in Mars body-fixed reference frame and spacecraft
     position relative to Mars in an inertial frame (any inertial frame 
     will do; J2000 was picked for convenience.)
     */
     subpt_c ( "Near point", "MARS", etprev, "NONE", "MRO", spointp, &alt );
     spkpos_c( "MRO", etprev, "J2000", "NONE", "MARS", sposp, &lt );

     for ( j = 1; j <= m; j++ )
     {
       /*
       Compute ET time of this step.
       */
       ettlp = etprev + j * tlp_substep;

       /*
       Compute the sub-spacecraft point location and spacecraft
       position at this ET time.
       */
       subpt_c ( "Near point", "MARS", ettlp, "NONE", "MRO", spoint, &alt );
       spkpos_c( "MRO", ettlp, "J2000", "NONE", "MARS", spos, &lt );

       /*
       Add the distance from the previous sub-spacecraft point to this
       sub-spacecraft point to the accumulating TLP length.
       */
       vsub_c( spoint, spointp, spointdiff );
       tlp_l = tlp_l + vnorm_c ( spointdiff );

       /*
       Add the angle between the previous s/c inertial position and
       this s/c inertial position to the accumulating TLP angle.
       */
       tlp_a = tlp_a + vsep_c ( spos, sposp );

       /*
       Re-assign the previous sub-spacecraft point and spacecraft
       position values.
       */
       vequ_c ( spoint, spointp );
       vequ_c ( spos,   sposp   );

     }
     
     /*
     Save computed TLP length and angle in the output arrays. Scale TLP
     angle to be in milliradians.
     */
     tlp_length[i-1] = tlp_l;
     tlp_angle[i-1]  = tlp_a * 1000.0;

   }
    
} /* End of odt_example */
